package beispiel;

import java.io.File;
import java.io.IOException;

import javax.xml.XMLConstants;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.xml.sax.SAXException;

import net.sf.saxon.s9api.Axis;
import net.sf.saxon.s9api.DocumentBuilder;
import net.sf.saxon.s9api.Processor;
import net.sf.saxon.s9api.QName;
import net.sf.saxon.s9api.SaxonApiException;
import net.sf.saxon.s9api.Serializer;
import net.sf.saxon.s9api.TeeDestination;
import net.sf.saxon.s9api.XdmDestination;
import net.sf.saxon.s9api.XdmNode;
import net.sf.saxon.s9api.XdmSequenceIterator;
import net.sf.saxon.s9api.XsltCompiler;
import net.sf.saxon.s9api.XsltExecutable;
import net.sf.saxon.s9api.XsltTransformer;

/**
 * Beispielhafte Umsetzung der in XGewerbeanzeige 1.3 (Entwurf) Abschnitt 3.2.2
 * zur Illustration der beschriebenen Validierungsschritte.
 * 
 * Die Pruefung der Schema-Konformitaet zu XStatistik erfolgt mit den Mitteln
 * des JAVA-API.
 * 
 * Die Ausfuehrung des GWA-Schematron-Validierers (XSLT-Skript) erfolgt mit
 * SAXON HE, http://www.saxonica.com/html/download/opensource.html,
 * API-Dokumentation:
 * http://www.saxonica.com/html/documentation/javadoc/net/sf/saxon/s9api/package-summary.html.
 * 
 * Die main-Methode erwartet zwei Verzeichnisnamen: - in dem ersten liegen die
 * zu validierenden XML-Dokumente, - in dem zweiten werden fuer die
 * Schema-validen Dokumente die Pruefergebnisse des GWA-Schematron-Validierers
 * abgelegt (Formatbeschreibung siehe Spezifikation XGewerbeanzeige).
 * 
 * Auf STDERR wird fuer jede gepruefte Datei ausgegeben, ob sie konform zu
 * DatML/RAW und der Liefervereinbarung ist.
 * 
 * THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 * @version 2017-05-17
 * @author Fabian.Buettner (KoSIT)
 *
 */
public class GWAValidator {

	public static String XSTATISTIK_SCHEMA = "resources/xstatistik-nachrichten.xsd";
	public static String VALIDATOR_XSLT_FILE = "resources/gwaschematronvalidierer.xsl";

	private SchemaFactory schemaFactory;
	private Schema schema;
	private Validator validator;
	
	private Processor processor;
	private XsltCompiler xsltCompiler;
	private XsltExecutable xsltValidator;
	private DocumentBuilder documentBuilder;

	public GWAValidator() throws SaxonApiException, SAXException {
		schemaFactory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
		schema = schemaFactory.newSchema(new StreamSource(XSTATISTIK_SCHEMA));
		processor = new Processor(false);
		xsltCompiler = processor.newXsltCompiler();
		xsltValidator = xsltCompiler.compile(new StreamSource(VALIDATOR_XSLT_FILE));
		documentBuilder = processor.newDocumentBuilder();
	}

	public boolean validateDocumentSchema(String inputFileName) throws IOException {
	    try {
	        validator = schema.newValidator();
	        StreamSource xmlFile = new StreamSource(inputFileName);
	        validator.validate(xmlFile);
	        return true;
	    } catch (SAXException e) {
	        // more on this error in e.getMessage() ...
	        return false;
	    }
	}

	public boolean validateDocumentLiefervereinbarung(String inputFileName, String outputFileName) throws SaxonApiException {
		XdmNode xdmSource = documentBuilder.build(new File(inputFileName));
		XsltTransformer transformer = xsltValidator.load();
		transformer.setInitialContextNode(xdmSource);
		XdmDestination resultAsTree = new XdmDestination();
		Serializer resultAsFile = processor.newSerializer(new File(outputFileName));
		resultAsFile.setOutputProperty(Serializer.Property.INDENT, "yes");
		transformer.setDestination(new TeeDestination(resultAsTree, resultAsFile));
		transformer.transform();
		XdmSequenceIterator failedAsserts = resultAsTree.getXdmNode().axisIterator(
				Axis.DESCENDANT, new QName("http://purl.oclc.org/dsdl/svrl", "failed-assert"));
		return !failedAsserts.hasNext();
	}

	public static void main(String[] args) throws SaxonApiException, SAXException, IOException {
		GWAValidator validierer = new GWAValidator();
		for (String inputFilename : args) {
			String outputFilename = inputFilename.replaceAll(".xml", ".report");
			if (validierer.validateDocumentSchema(inputFilename)) {
				if (validierer.validateDocumentLiefervereinbarung(inputFilename, outputFilename)) {
					System.err.println(inputFilename + ": konform zu DatML/RAW und Liefervereinbarung.");
				} else {
					System.err.println(inputFilename + ": nicht konform zur Liefervereinbarung.");
				}
			} else {
				System.err.println(inputFilename + ": nicht konform zu DatML/RAW.");
			}
		}
	}
}
